# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry, conf
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose 
#------------------------------------------------------------------------------------
#
#	Calculates the instrumental magnitudes of asteroids and variable stars over a series of images.
#	This script is very similar to "photometry_run.py"
#	It does not try to optimise the final data. Rather, it computes the SNR from the CCD equation for each magnitude value.
#	For more explanations see: http://www.aspylib.com/doc/using_photometry_script.html
#
#------------------------------------------------------------------------------------
#	Input parameters (basic)
#------------------------------------------------------------------------------------

#--- calibrated images ---
Folder = u"C:\\images\\"
Imagelist = [Folder + "images-calibrated-" + str(i+1) + ".fit" for i in range(100)]

#radii for aperture photometry (pixels)
sizes = [[2,8,40],[2.5,8,40],[3,8,40],[3.5,8,40],[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40]]
#sizes = [[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40],[6.5,10,45],[7,10,45],[7.5,12,45],[8,12,45],[8.5,14,50],[9,14,50]]
#sizes = [[8,16,45],[8.5,16,50],[9,16,50],[9.5,16,55],[10,18,50],[10.5,18,50],[11,18,50],[11.5,18,50],[12,20,50]]

#--- names of output files ---
generic_name = Folder + 'test_'

#--- flag to shortcut image shift calculations ---
flag_calc = True	#if True, checks images and calculates shifts. If False, read results from file
					
#--- CCD camera parameters ---
CCD = {}
CCD['ROnoise'] = 25.
CCD['gain'] = 4.85	#in electrons/ADU
CCD['flatgain'] = 1. #/1.2

#------------------------------------------------------------------------------------
#	Input parameters (advanced)
#------------------------------------------------------------------------------------

#--- semi-size of square areas used for star max detection and star fitting ---
psf_fit_area = 8			#typically a few times (2-3) the star FWHM.
max_detection_area = 2		#max_detection_area = 2 is recommended. Can be SLIGHTLY increased for very large stars (FWHM >5 pixels).

#--- photometry method ---
#method = 'select'
#method = 'maximum'
method = 'centroid'

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_selected_stars.pdf"
outputname4 = generic_name + "04_inst_mag_and_SNR.txt"

#####################################################################################
#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
print
print '*********************************************************************'
print '      SCRIPT FOR APERTURE PHOTOMETRY'
print '*********************************************************************'

#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

if flag_calc:
	#------ checking quality of measured frames ------
	print
	print '---------------------------------------------------------------------'
	print '      Step 1 : checking the input images'
	print '---------------------------------------------------------------------'
	print
	Imagelist2 = photometry.check_pictures_quality(Imagelist, outputname1)

	#------ calculating shifts ------
	print
	print '---------------------------------------------------------------------'
	print '      Step 2 : calculation of image shifts'
	print '---------------------------------------------------------------------'
	print
	Imagelist3, shifts = photometry.calculate_shifts(Imagelist2, outputname2, detection_area = psf_fit_area, nb_stars = 12)
else:
	Imagelist3, shifts = photometry.load_shifts(outputname2)
	plt.ion()
	plt.plot(shifts[:,0], 'b')
	plt.plot(shifts[:,1], 'g')
	plt.grid()
	plt.title('Image shifts: Vertical (blue), Horizontal (green)')
	plt.xlabel('Image number')
	plt.ylabel('Image shifts (pixels)')
	astro.maximize_window(conf.cmd)
	sx, sy = astro.get_window_size(conf.cmd)
	plt.draw()	
	fx, fy = astro.get_window_position("Figure 1")
	astro.move_window("Figure 1", [sx, fy])
	print
	raw_input("Press Enter to continue")
	plt.close()
	
#------------------------------------------------------------------------------------
#	calculates instrumental magnitudes
#------------------------------------------------------------------------------------

#------ date information ------
dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))

#------ select stars ------
print
print '---------------------------------------------------------------------'
print '      Step 3 : selecting targets for photometry'
print '---------------------------------------------------------------------'
print
if method=='select':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = 0, method = method)
elif method=='maximum':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = max_detection_area, method = method)
elif method=='centroid':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = psf_fit_area, method = method)

#------ perform aperture photometry ------
print
print '---------------------------------------------------------------------'
print '      Step 4 : calculating aperture photometry'
print '---------------------------------------------------------------------'
print
mag, SNR = photometry.make_photometrystats(Imagelist3, sizes, xy, CCD, method=method)
photometry.write_magnitudes_and_SNR(Imagelist3, outputname4, names, xy, dateobs, sizes, mag, SNR)	

raw_input("Press ENTER to close")