# -*- coding: iso_8859_1 -*-

import numpy as np
import math

def air_mass(alpha, delta, lat, long, JJ):
	"""
	---------------------
	Purpose
	Calculates air mass ratio for specific location, time and sky coordinates of an object 
	---------------------
	Inputs
	* alpha (float) = right ascension, in degrees.
	* delta (float) = declination, in degrees.
	* lat (float) = observing geodetic latitude, in degrees.
	* long (float) = observing longitude, positive East of Greenwich, in degrees.
	* JJ (float) = julian date for which air mass ratio must be calculated
	---------------------
	Output (float) = air mass ratio
	---------------------
	"""

	#alpha, delta, lat, long in degrees
	#long is positive East of Greenwich
	
	#sideral time at Greenwich, 0h UT [in hours]
	T = (JJ-2415020.)/36525.
	theta0 = 6.6460656 + 2400.051262*T + 0.00002581*T*T
	
	#sideral time at Greenwich [in degrees]
	theta = np.mod(theta0 + 1.002737908 * (JJ-np.floor(JJ) + 0.5) * 24., 24.) * 15.
	
	#angle horaire [in degrees]
	H = np.mod(theta + long - alpha, 360.)
	
	#elevation [in degrees]
	k = math.pi/180.
	elevation = (1./k)*np.arcsin(np.sin(lat*k)*np.sin(delta*k)+np.cos(lat*k)*np.cos(delta*k)*np.cos(H*k))
	
	#air mass ratio
	dist = 90.-elevation
	air_mass = (1./np.cos(dist*k))*(1.-0.0012*(np.tan(dist*k)**2.))
	
	return air_mass

def hours_to_decdegrees(hours, sep=':'):
	"""
	---------------------
	Purpose
	Transforms string 'hours:minutes:seconds' to decimal number in degrees
	The default separating character is ':' but can be any other non-numeric character (e.g. ' ').
	---------------------
	Inputs
	* hours (string) = string to be converted
	No minus sign is allowed in the input string. Allowed formats are:
		'13:34:12.45'
		'13:34:12'
	* sep (character) = separating character. Optional input, default value is ':'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = hours.split(sep)
	return 15.*(np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.)
	
def degrees_to_decdegrees(degrees, sep=':'):
	"""
	---------------------
	Purpose
	Transforms string 'degrees:arcmin:arcsec' to decimal number in degrees
	The default separating character is ':' but can be any other non-numeric character (e.g. ' ').
	---------------------
	Inputs
	* degrees (string) = string to be converted
	A plus or minus sign is allowed for the first character. Allowed formats are:
		'-06:34:23.1'
		'-06:34:23'
		'+06:34:23.1'
		'+06:34:23'
		'06:34:23.1'
		'06:34:23'
	* sep (character) = separating character. Optional input, default value is ':'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = degrees.split(sep)
	if values[0][0] == '-':
		result = -(np.abs(np.float(values[0]))+np.float(values[1])/60.+np.float(values[2])/3600.)
	else:
		result = np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.
	return result

def dates_to_juliandays(dates):
	"""
	---------------------
	Purpose
	Converts a list of values representing dates and provided in the format "YYYYMMDD.dddddd", into a list of corresponding Julian dates
	---------------------
	Inputs
	* dates (list) = list of floats, in the form [d1, d2, ... dn]. Each float number is a date, with a format equivalent to "20110321.981322" for 21.981322 March 2011.
	---------------------
	Output (list) = list of floats, in the form [j1, j2, ... jn]. Each float number "ji" is the Julian date that corresponds to "di".
	---------------------
	"""

	result = []
	for dd in dates:

		year = float(str(dd)[0:4])
		month = float(str(dd)[4:6])
		day = float(str(dd)[6:])
		
		if month<=2:
			month = month+12
			year = year-1
		
		a = np.trunc(year/100.)
		b = 2. - a + np.trunc(a/4.)
			
		result.append(np.trunc(365.25*year)+np.trunc(30.6001*(month+1.))+day+1720994.5+b)
	return result

def juliandays_to_dates(juliandates):
	"""
	---------------------
	Purpose
	Converts a list of Julian dates into a list of values representing dates and expressed in the format "YYYYMMDD.dddddd"
	---------------------
	Inputs
	* juliandates (list) = list of floats, in the form [j1, j2, ... jn]. Each float number "ji" is a Julian date.
	---------------------
	Output (list) = list of floats, in the form [d1, d2, ... dn]. Each float number "di" is a date, with a format equivalent to "20110321.981322" for 21.981322 March 2011, that corresponds to the Julian date "ji".
	---------------------
	"""

	result = []
	for jj in juliandates:
		Z = float(int(jj+0.5))		#integer part of jj+0.5
		F = jj+0.5-Z				#fractional part
		if Z<2299161.:
			A=Z
		else:
			alpha = float(int((Z-1867216.25)/36524.25))
			A = Z+1.+alpha-float(int(alpha/4.))
		B = A + 1524.
		C = float(int((B-122.1)/365.25))
		D = float(int(365.25*C))
		E = float(int((B-D)/30.6001))
		day = B-D-int(30.6001*E)+F
		if E<14:
			month = E-1
		elif (E==14)or(E==15):
			month = E-13
		if month>2:
			year = C-4716.
		elif (month==1)or(month==2):
			year = C-4715.
			
		date = "%04d"%year+"%02d"%month+"%02d"%int(day)+("%.6g"%(day-int(day)))[1:]
		result.append(date)
	return result
	
def decdegrees_to_hours(angle, sep=':'):
	"""
	---------------------
	Purpose
	Transforms a decimal number in degrees to a string 'hours:minutes:seconds'
	The default separating character is ':' but can be any other non-numeric character (e.g. ' ').
	---------------------
	Inputs
	* angles (float) = number to be converted
	* sep (character) = separating character. Optional input, default value is ':'
	---------------------
	Output (string) = result returned with format 'hours:minutes:seconds'
	---------------------
	"""
	value = np.mod(angle,360.)/15.
	hours = np.int(value)
	minutes = np.int((value-hours)*60.)
	seconds = (value-hours-minutes/60.)*3600.
	hrs = '%(hrs)02d' % {'hrs': hours}
	min = '%(min)02d' % {'min': minutes}
	sec = '%(sec)05.2f' % {'sec': seconds}

	return hrs+sep+min+sep+sec

def decdegrees_to_degrees(angle, sep=':'):
	"""
	---------------------
	Purpose
	Transforms a decimal number in degrees to a string 'degrees:arcminutes:arcseconds'
	The default separating character is ':' but can be any other non-numeric character (e.g. ' ').
	---------------------
	Inputs
	* angles (float) = number to be converted
	* sep (character) = separating character. Optional input, default value is ':'
	---------------------
	Output (string) = result returned with format 'degrees:arcminutes:arcseconds'
	---------------------
	"""
	sign = np.sign(angle)
	degrees = np.int(np.abs(angle))
	arcminutes = np.int((np.abs(angle)-degrees)*60.)
	arcseconds = (np.abs(angle)-degrees-arcminutes/60.)*3600.
	if sign>=0:
		header = '+'
	else:
		header = '-'
	deg = '%(deg)02d' % {'deg': degrees}
	arcmin = '%(arcmin)02d' % {'arcmin': arcminutes}
	arcsec = '%(arcsec)04.1f' % {'arcsec': arcseconds}

	return header+deg+sep+arcmin+sep+arcsec

def greatcircledistance(alpha1, delta1, alpha2, delta2):
	"""
	---------------------
	Purpose
	Computes the angular distance, in degrees, between two points 1 and 2, given their positions in decimal degrees.
	---------------------
	Inputs
	* alpha1 (float) = right ascension of point 1 (deg)
	* delta1 (float) = declination of point 1 (deg)
	* alpha2 (float) = right ascension of point 2 (deg)
	* delta2 (float) = declination of point 2 (deg)
	---------------------
	Output (float) = angular distance (deg)
	---------------------
	"""
	k = np.pi/180.
	return (180./np.pi)*np.arccos(np.sin(k*delta1)*np.sin(k*delta2)+np.cos(k*delta1)*np.cos(k*delta2)*np.cos(k*(alpha2-alpha1)))
	
def haversine(alpha1, delta1, alpha2, delta2):
	"""
	---------------------
	Purpose
	Computes the angular distance, in degrees, between two points 1 and 2, given their positions in decimal degrees.
	This function is using the Haversine method, which is accurate for very small distances.
	---------------------
	Inputs
	* alpha1 (float) = right ascension of point 1 (deg)
	* delta1 (float) = declination of point 1 (deg)
	* alpha2 (float) = right ascension of point 2 (deg)
	* delta2 (float) = declination of point 2 (deg)
	---------------------
	Output (float) = angular distance (deg)
	---------------------
	"""
	k = np.pi/180.
	haver = (1.-np.cos(k*(delta2-delta1))) + np.cos(k*delta1)*np.cos(k*delta2)*(1.-np.cos(k*(alpha2-alpha1)))
	return (180./np.pi)*np.arccos(1.-haver)