# -*- coding: iso_8859_1 -*-

#import matplotlib
#matplotlib.use('GTKAgg') 

import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib.widgets import Slider

import numpy as np
import time, warnings
import stats


def display(data, title=''):
	"""
	---------------------
	Purpose
	Displays the data of a FITS image.
	The display window allows to zoom in, zoom out, adjust visualisation thresholds, and save the displayed image in an image file.
	---------------------
	Inputs
	* data (2D Numpy array) = the data of one FITS image, as obtained for instance by reading a list containing a single FITS file with the function astro.get_imagedata(), or by reading a list of several images with astro.get_imagedata() and then extracting the data of a single image from the returned data.
	The function astro.display() works with any 2D Numpy array.
	* title (string) = if specified, defines the title of the window where the image is displayed. If not specified or equal to an empty string, the title is automatically created by Matplotlib (and then it is 'Figure 1' for the 1st figure, etc).
	---------------------
	Outputs = fig, ax
	* fig (matplotlib.figure.Figure instance) = the figure that contains the displayed image
	* ax (matplotlib.axes.Axes instance) = the figure element that corresponds to the displayed image (other figure elements are the sliders)
	---------------------
	"""
	data = np.array(data,dtype=np.int)
	med = stats.get_median(data)[0]
	dsig = np.std(data.flatten())
	
	plt.ion()
	fig = plt.figure()
	ax = plt.subplot(111)
	plt.subplots_adjust(bottom=0.2)

	h = plt.imshow(data, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
	
	#------ defining sliders ------
	axmini = plt.axes([0.1, 0.05, 0.8, 0.03])
	axmaxi  = plt.axes([0.1, 0.1, 0.8, 0.03])

	smini = Slider(axmini, 'min', med-5*dsig, med+20*dsig, dragging=False, valinit=med-0.5*dsig, valfmt='%1.0f')
	smaxi = Slider(axmaxi, 'max', med-5*dsig, med+20*dsig, dragging=False, valinit=med+2*dsig, valfmt='%1.0f')

	smini.poly.set_color('0.5')
	smaxi.poly.set_color('0.5')
	def update(val):
		mini = smini.val
		maxi = smaxi.val
		if mini<=maxi:
			dat = data
			h.set_data(dat)
			h.set_clim(vmin=smini.val, vmax=smaxi.val)
		else:
			dat = maxi+mini-data
			h.set_data(dat)
			h.set_clim(vmin=smaxi.val, vmax=smini.val)
			
		plt.draw()

	smini.on_changed(update)
	smaxi.on_changed(update)

	if title!='':
		fig.canvas.set_window_title(title) 
	plt.draw()
	return fig, ax


def select(fig, ax, nb, color='none', marker='o', size=50): 
	"""
	---------------------
	Purpose
	Return the (x,y) coordinates of selected pixels on a FITS image displayed with the astro.display() function.
	The selection is made by placing the cursor on a specific pixel, and then making a mouse right-click.
	---------------------
	Inputs
	* fig (matplotlib.figure.Figure instance) = the figure that contains the displayed image
	* ax (matplotlib.axes.Axes instance) = the figure element that corresponds to the displayed image (other figure elements are the sliders)
	* nb (integer) = the number of pixels to be selected. If equal to -1, an arbitrary number of pixels can be selected: the selection is finished with a mouse right-click outside of the displayed image.
	* color (string) = optional argument, if set to a Matplotlib color then a Marker is placed on the image on each selected pixel
	* marker (string) = optional argument, gives the shape of the marker
	* size (integer) = optional argument, gives the size of the marker
	The inputs "color", "marker" and "size" can take any value that is accepted by the Matplotlib.pyplot.scatter() function for the arguments (respectively): c, marker, s. For more information see the Matplotlib help (http://matplotlib.sourceforge.net/api/pyplot_api.html?highlight=scatter#matplotlib.pyplot.scatter)
	---------------------
	Output (list) = has the form [[x1,y1],[x2,y2],...,[xn,yn]]. The list [xi,yi] gives the coordinates of the ith selected pixel.
	---------------------
	"""
	xy = [] 
	cid = 0 
	warnings.simplefilter("ignore") 
	global i 
	i=0 

	#------------ 
	def on_click(event): 
		global i 
		if event.inaxes is not None: 
			if event.button==3: 
				xy.append([int(round(event.ydata)),int(round(event.xdata))]) 
				i = i+1 
				if color != 'none': 
					limits = ax.axis()
					ax.scatter(xy[-1][1],xy[-1][0],s=size, marker=marker, edgecolors=color, facecolor='none') 
					ax.annotate(str(i), xy=(xy[-1][1]+10, xy[-1][0]+10),  xycoords='data', color=color) 
					ax.axis(limits)
					plt.draw() 
				print 'vert=',xy[-1][0],'horiz=',xy[-1][1] 
				if len(xy)==nb: 
					fig.canvas.mpl_disconnect(cid) 
					fig.canvas.stop_event_loop_default() 
		else: 
			if nb<0: 
				if event.button==3: 
					fig.canvas.mpl_disconnect(cid) 
					fig.canvas.stop_event_loop_default() 

	cid = fig.canvas.mpl_connect('button_press_event', on_click) 
	fig.canvas.start_event_loop_default() 

	warnings.resetwarnings() 
	time.sleep(0.25) 
	return xy
