# -*- coding: iso_8859_1 -*-

import numpy as np
import stats
import scipy.ndimage as sn


def detect_peaks(image, detection_area = 2):
	"""
	---------------------
	Purpose
	Takes an image and detects the peaks using a local maximum filter.
	Returns a boolean mask of the peaks (i.e. 1 when the pixel's value is a local maximum, 0 otherwise)
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* detection_area (integer) = optional argument. The local maxima are found in a square neighborhood with size (2*detection_area+1)*(2*detection_area+1). Default value is detection_area = 2.
	---------------------
	Output (2D Numpy array) = Numpy array with same size as the input image, with 1 at the image local maxima, 0 otherwise.
	---------------------
	"""
	# define an 8-connected neighborhood --> image dimensionality is 2 + there are 8 neighbors for a single pixel
	neighborhood = np.ones((1+2*detection_area,1+2*detection_area))
	#apply the local maximum filter; all pixel of maximal value in their neighborhood are set to 1
	local_max = sn.maximum_filter(image, footprint=neighborhood)==image
	#local_max is a mask that contains the peaks we are looking for, but also the background. In order to isolate the peaks we must remove the background from the mask. We create the mask of the background
	background = (image==0)
	#a little technicality: we must erode the background in order to successfully subtract it from local_max, otherwise a line will appear along the background border (artifact of the local maximum filter)
	eroded_background = sn.binary_erosion(background, structure=neighborhood, border_value=1)
	#we obtain the final mask, containing only peaks, by removing the background from the local_max mask
	detected_peaks = local_max - eroded_background
	return detected_peaks

	# http://stackoverflow.com/questions/3684484/peak-detection-in-a-2d-array/3689710#3689710     
	# http://www.scipy.org/doc/api_docs/SciPy.ndimage.morphology.html
	# http://www.scipy.org/doc/api_docs/SciPy.ndimage.filters.html
	# http://www.scipy.org/doc/api_docs/SciPy.ndimage.morphology.html

def detect_stars(image, threshold, detection_area = 2, saturation = [False,0], margin = 5):
	"""
	---------------------
	Purpose
	Takes an image and detects all objects above a certain threshold.
	Returns a boolean mask giving the objects positions (i.e. 1 when the pixel's value is at an object's center, 0 otherwise)
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* threshold (integer) = the objects are only detected if their maximum signal is above threshold.
	* detection_area (integer) = optional argument. The object's local maxima are found in a square neighborhood with size (2*detection_area+1)*(2*detection_area+1). Default value is detection_area = 2.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, the objects with maximum signal >= saturation[1] are disregarded.
	* margin (integer or list) = optional argument. The default value is 5. The objects having their center closer than margin pixels from the image border are disregarded. Margin can also be a list of 4 integers, in which case it represents the top, bottom, right, left margins respectively.
	---------------------
	Output (2D Numpy array) = Numpy array with same size as the input image, with 1 at the objects centers, 0 otherwise.
	---------------------
	"""
	sigmafilter = 1

	#--- threshold ---
	imageT = image>=threshold

	#--- smooth threshold, mask and filter initial image ---
	neighborhood = sn.generate_binary_structure(2,1)
	imageT = sn.binary_erosion(imageT, structure=neighborhood, border_value=1)
	imageT = sn.binary_dilation(imageT, structure=neighborhood, border_value=1)
	imageF = sn.filters.gaussian_filter(imageT*image,sigmafilter)

	#--- find local max ---
	peaks = detect_peaks(imageF, detection_area = detection_area)
	if isinstance(margin,list):
		peaks[-margin[0]:,:]=0
		peaks[:margin[1],:]=0
		peaks[:,-margin[2]:]=0
		peaks[:,:margin[3]]=0
	else:
		peaks[-margin:,:]=0
		peaks[:margin,:]=0
		peaks[:,-margin:]=0
		peaks[:,:margin]=0

	#--- removes too bright stars (possibly saturated) ---
	if saturation[0]:
		peaks = peaks * (image<saturation[1])
	
	return peaks

	
def find_stars(image, nb_stars, excluded = [], included = [], detection_area = 2, saturation = [False,0], margin = 5, text_display = True):
	"""
	---------------------
	Purpose
	Takes an image and returns the approximate positions of the N brightest stars, N being specified by the user.
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* nb_stars (integer) = the number of stars requested.
	* excluded (list) = optional argument. List of 4 elements lists, with the form [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]]. Each element is a small part of the image that will be excluded for the star detection.
	This optional parameter allows to reject problematic areas in the image (e.g. saturated star(s), artefact(s), moving object(s)..)
	* included (list) = optional argument. List of 4 integers, with the form [x1,x2,y1,y2]. It defines a subpart of the image where all the stars will be detected.
	Both excluded and included can be used simultaneously.
	* detection_area (integer) = optional argument. The object's local maxima are found in a square neighborhood with size (2*detection_area+1)*(2*detection_area+1). Default value is detection_area = 2.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, the objects with maximum signal >= saturation[1] are disregarded.
	* margin (integer) = optional argument. The objects having their center closer than margin pixels from the image border are disregarded.
	* text_display (boolean) = optional argument. If True then some information about the star detection is printed in the console.
	---------------------
	Output (2D Numpy array) = Numpy array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate x and y positions for each star.
	---------------------
	"""

	if text_display:
		print
		print "--- detecting stars ---"
		print "threshold, nb of detected stars:"

	#--- included ---
	if len(included)>0:
		x0, x1, y0, y1 = included
		im = image[x0:x1,y0:y1]
	else:
		im = image
		x0, y0 = [0,0]
	
	#--- get subimage information ---
	maxi = np.max(im.flatten())
	median = stats.get_median(im)[0]

	#--- searching threshold ---
	th = median + 0.1*maxi
	peaks = detect_stars(im, th, detection_area = detection_area, saturation = saturation, margin = margin)
	for area in excluded:
		x2,x3,y2,y3 = area
		peaks[max(x2-x0,0):max(x3-x0,0),max(y2-y0,0):max(y3-y0,0)] = 0
	x, y = np.where(peaks==1)
	x = x + x0
	y = y + y0
	nb = len(x)
	if text_display:
		print th, nb

	iter = 0
	while nb<nb_stars:
		iter +=1
		th = th*0.1+median*0.9
		peaks = detect_stars(im, th, detection_area = detection_area, saturation = saturation, margin = margin)
		for area in excluded:
			x2,x3,y2,y3 = area
			peaks[max(x2-x0,0):max(x3-x0,0),max(y2-y0,0):max(y3-y0,0)] = 0
		x, y = np.where(peaks==1)
		x = x + x0
		y = y + y0
		nb = len(x)
		if text_display:
			print th, nb
		if iter>10:
			raise Exception("error in aspylib.astro.find_stars(), star detection does not converge")

	xy = np.vstack([x,y]).transpose()
	maxi = image[list(x),list(y)]
	xy = xy[np.flipud(np.argsort(maxi)),:]

	if text_display:
		print "number of detected stars =",nb_stars

	return xy[:nb_stars,:]
