# -*- coding: iso_8859_1 -*-

import numpy as np
import sys

def project_stars(alpha, delta, alpha0, delta0):
	"""
	---------------------
	Purpose
	Transform a list of (alpha, delta) star coordinates into (pv, ph) cartesian coordinates using an orthographic projection, which is conformal (=does not change the shapes).
	pv stands for "projected vertical" and ph for "projected horizontal".
	The transformation is described in:
	N.Kaiser, G.Wilson, G.Luppino, H.Dahle "A photometric study of the supercluster MS0302 with the UH8K CCD camera - image processing and object catalogs"
	arXiv:astro-ph/9907229v1 (16 Jul 1999)
	---------------------
	Inputs
	* alpha (Numpy array) = right ascension, in degrees, for the stars to be transformed.
	* delta (Numpy array) = declination, in degrees, for the stars to be transformed.
	* alpha0 (float) = right ascension, in degrees, for the center of the field of view.
	* delta0 (float) = declination, in degrees, for the center of the field of view.
	The point with coordinates (alpha0, delta0) is the point of the projection plane that is tangent to the unit sphere.
	---------------------
	Output (Numpy array) = Numpy array such that: Output[:,0] = vertical projected coordinate, Output[:,1] = horizontal projected coordinate.
	---------------------
	"""
		
	alpha0 = alpha0*np.pi/180.
	delta0 = delta0*np.pi/180.

	axis = [np.cos(delta0)*np.cos(alpha0), np.cos(delta0)*np.sin(alpha0), np.sin(delta0)]
	pvert = [-np.sin(delta0)*np.cos(alpha0), -np.sin(delta0)*np.sin(alpha0), np.cos(delta0)]
	phoriz = [np.sin(alpha0), -np.cos(alpha0), 0.0]
	
	stars_X = np.cos(delta*np.pi/180.)*np.cos(alpha*np.pi/180.)
	stars_Y = np.cos(delta*np.pi/180.)*np.sin(alpha*np.pi/180.)
	stars_Z = np.sin(delta*np.pi/180.)
	
	dot_axis = stars_X*axis[0] + stars_Y*axis[1] + stars_Z*axis[2]
	dot_pvert = stars_X*pvert[0] + stars_Y*pvert[1] + stars_Z*pvert[2]
	dot_phoriz = stars_X*phoriz[0] + stars_Y*phoriz[1] + stars_Z*phoriz[2]
	
	return np.vstack([1.e6*(2.*dot_pvert) / (1.+dot_axis), 1.e6*(2.*dot_phoriz) / (1.+dot_axis)]).transpose()

def inverse_project_stars(pv, ph, alpha0, delta0):
	"""
	---------------------
	Purpose
	Perform the inverse of the transformation made by the function astrometry.project_stars()
	---------------------
	Inputs
	* pv (Numpy array) = vertical projected coordinate, in degrees, for the stars to be transformed.
	* ph (Numpy array) =  horizontal projected coordinate, for the stars to be transformed.
	* alpha0 (float) = right ascension, in degrees, for the center of the field of view.
	* delta0 (float) = declination, in degrees, for the center of the field of view.
	The point with coordinates (alpha0, delta0) is the point of the projection plane that is tangent to the unit sphere.
	---------------------
	Output (Numpy array) = Numpy array such that: Output[:,0] = right ascension (in degrees), Output[:,1] = declination (in degrees).
	---------------------
	"""

	alpha0 = alpha0*np.pi/180.
	delta0 = delta0*np.pi/180.

	axis = [np.cos(delta0)*np.cos(alpha0), np.cos(delta0)*np.sin(alpha0), np.sin(delta0)]
	pvert = [-np.sin(delta0)*np.cos(alpha0), -np.sin(delta0)*np.sin(alpha0), np.cos(delta0)]
	phoriz = [np.sin(alpha0), -np.cos(alpha0), 0.0]

	pv = pv*1.e-6
	ph = ph*1.e-6
	
	coef1 = 4.*pv / (4.+pv**2.+ph**2.)
	coef2 = 4.*ph / (4.+pv**2.+ph**2.)
	coef3 = (4.-pv**2.-ph**2.) / (4.+pv**2.+ph**2.)
	
	stars_X = coef1*pvert[0] + coef2*phoriz[0] + coef3*axis[0]
	stars_Y = coef1*pvert[1] + coef2*phoriz[1] + coef3*axis[1]
	stars_Z = coef1*pvert[2] + coef2*phoriz[2] + coef3*axis[2]
	
	alpha = (180./np.pi)*np.arctan2(stars_Y, stars_X)
	alpha = (alpha<0.)*(alpha+360.) + (alpha>=0.)*alpha
	delta = (180./np.pi)*np.arcsin(stars_Z)

	return np.vstack([alpha, delta]).transpose()
