# -*- coding: iso_8859_1 -*-

from aspylib import astro
import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage as sn
#import itertools as it

def find_rotation_and_scaling(stars1, stars2, resolution=512, text_display=True, plotting=True):
	"""
	---------------------
	Purpose
	Calculate the rotation and scaling needed to match two lists of star positions with a linear transformation (rotation, scaling, translation). The translation is not calculated.
	The rotation and scaling are obtained with a cross-correlation method (see N. Kaiser, G. Wilson, G. Luppino, H. Dahle, "A photometric study of the supercluster MS0302 with the UH8K CCD camera: image processing and object catalogs", arXiv:astro-ph/9907229v1, 16 Jul 1999).
	---------------------
	Inputs
	* stars1 (2D Numpy array) = first list of star positions. stars1[:,0] corresponds to the X (vertical) coordinates and stars1[:,1] to the Y (horizontal) coordinates.
	* stars2 (2D Numpy array) = second list of star positions, with the same structure as stars1.
	* resolution (integer) = optional parameter, default value is 512. This is the size in pixels of the artificial square images that are created and Fourier-transformed to calculate a cross-correlation.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	* plotting (boolean) = optional argument. If equal to True (default), the script is paused to display a plot showing the matching between the two lists of star positions. If equal to False no plot is shown.
	---------------------
	Outputs = dphi, scaling
	* dphi (float) = rotation angle, expressed in radians, measured clockwise (from X=vertical to Y=horizontal), to transform the positions stars1 into the positions stars2.
	* scaling (float) = scaling factor, needed to transform the positions stars1 into the positions stars2.
	---------------------
	"""
	#*********************************************************************************************************
	#		(1) looks for scaling and rotation between catalog and image
	#*********************************************************************************************************

	list_1 = []
	for stA in range(np.shape(stars1)[0]):
		for stB in range(stA+1,np.shape(stars1)[0]):
			xA = stars1[stA,0]
			yA = stars1[stA,1]
			xB = stars1[stB,0]
			yB = stars1[stB,1]
			phi = np.mod(np.arctan2(yB-yA, xB-xA),np.pi)
			logd = 0.5*np.log((yB-yA)**2.+(xB-xA)**2)
			list_1.append([phi,logd])
	list_1 = np.array(list_1)

	list_2 = []
	for stA in range(np.shape(stars2)[0]):
		for stB in range(stA+1,np.shape(stars2)[0]):
			xA = stars2[stA,0]
			yA = stars2[stA,1]
			xB = stars2[stB,0]
			yB = stars2[stB,1]
			phi = np.mod(np.arctan2(yB-yA, xB-xA),np.pi)
			logd = 0.5*np.log((yB-yA)**2.+(xB-xA)**2)
			list_2.append([phi,logd])		
	list_2 = np.array(list_2)

	#--- other possibility ---
	#list_1 = np.array(map(lambda xy: [np.mod(np.arctan2(xy[1][1]-xy[0][1], xy[1][0]-xy[0][0]),np.pi), 0.5*np.log((xy[1][1]-xy[0][1])**2.+(xy[1][0]-xy[0][0])**2)],list(it.combinations(stars1,2))))
	#list_2 = np.array(map(lambda xy: [np.mod(np.arctan2(xy[1][1]-xy[0][1], xy[1][0]-xy[0][0]),np.pi), 0.5*np.log((xy[1][1]-xy[0][1])**2.+(xy[1][0]-xy[0][0])**2)],list(it.combinations(stars2,2))))
	#-------------------------
	
	min1 = np.min(list_1[:,1])
	min2 = np.min(list_2[:,1])
	max1 = np.max(list_1[:,1])
	max2 = np.max(list_2[:,1])
	b1 = np.floor(min(min1,min2))
	b2 = np.ceil(max(max1,max2))
	b1 = b1-(b2-b1)
	b2 = b2+(b2-b1)

	im1 = np.zeros((resolution,resolution))
	for xy in list_1:
		im1[np.round(xy[0]*(resolution-1)/np.pi),np.round((xy[1]-b1)*(resolution-1)/(b2-b1))] = 1.
		
	im2 = np.zeros((resolution,resolution))
	for xy in list_2:
		im2[np.round(xy[0]*(resolution-1)/np.pi),np.round((xy[1]-b1)*(resolution-1)/(b2-b1))] = 1.
	
	imag1 = sn.filters.gaussian_filter(im1,1.0)
	imag2 = sn.filters.gaussian_filter(im2,1.0)
	
	#astro.display(imag1*100)
	#astro.display(imag2*100)
	
	dphi, dlogd = astro.find_shift_crosscorrelation(imag1, imag2)
	dphi = dphi * (np.pi/(resolution-1)) #* (180./np.pi)

	dlogd = dlogd * ((b2-b1)/(resolution-1))
	scaling = np.exp(dlogd)

	if text_display:
		print "rotation (rad):", dphi
		print "rotation (deg):", dphi*180./np.pi
		print "log(scaling):", dlogd
		print "scaling:", scaling
	
	if plotting:
		plt.ion()
		plt.figure()
		plt.scatter(list_1[:,0]+dphi,list_1[:,1]+dlogd,s=20)
		plt.scatter(list_2[:,0],list_2[:,1],s=20,marker='+',color='r')
		plt.grid()
		plt.title("phi-log(d) diagram")
		plt.draw()

	return dphi, scaling
	
def find_translation(stars1, stars2, resolution=512, text_display=True, plotting=True):
	"""
	---------------------
	Purpose
	Calculate the translation needed to match two lists of star positions.
	This function can be called after astrometry.find_rotation_and_scaling() to find the translation that remains unknown.
	---------------------
	Inputs
	* stars1 (2D Numpy array) = first list of star positions. stars1[:,0] corresponds to the X (vertical) coordinates and stars1[:,1] to the Y (horizontal) coordinates.
	* stars2 (2D Numpy array) = second list of star positions, with the same structure as stars1.
	* resolution (integer) = optional parameter, default value is 512. This is the size in pixels of the artificial square images that are created and Fourier-transformed to calculate a cross-correlation.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	* plotting (boolean) = optional argument. If equal to True (default), the script is paused to display a plot showing the matching between the two lists of star positions. If equal to False no plot is shown.
	---------------------
	Outputs = dx, dy
	* dx (float) = translation to be applied on X (vertical) coordinate to transform the positions stars1 into the positions stars2.
	* dy (float) = same as dx but for Y (horizontal) coordinate.
	---------------------
	"""
	#*********************************************************************************************************
	#		(2) looks for remaining translation between catalog and image
	#*********************************************************************************************************

	#--- x boundaries ---
	min1 = np.min(stars1[:,0])
	min2 = np.min(stars2[:,0])
	max1 = np.max(stars1[:,0])
	max2 = np.max(stars2[:,0])
	bx1 = np.floor(min(min1,min2))
	bx2 = np.ceil(max(max1,max2))
	bx1 = bx1-(bx2-bx1)
	bx2 = bx2+(bx2-bx1)

	#--- y boundaries ---
	min1 = np.min(stars1[:,1])
	min2 = np.min(stars2[:,1])
	max1 = np.max(stars1[:,1])
	max2 = np.max(stars2[:,1])
	by1 = np.floor(min(min1,min2))
	by2 = np.ceil(max(max1,max2))
	by1 = by1-(by2-by1)
	by2 = by2+(by2-by1)

	im1 = np.zeros((resolution,resolution))
	for xy in stars1:
		im1[np.round((xy[0]-bx1)*(resolution-1)/(bx2-bx1)),np.round((xy[1]-by1)*(resolution-1)/(by2-by1))] = 1.
		
	im2 = np.zeros((resolution,resolution))
	for xy in stars2:
		im2[np.round((xy[0]-bx1)*(resolution-1)/(bx2-bx1)),np.round((xy[1]-by1)*(resolution-1)/(by2-by1))] = 1.
		
	imag1 = sn.filters.gaussian_filter(im1,1.0)
	imag2 = sn.filters.gaussian_filter(im2,1.0)

	dx, dy = astro.find_shift_crosscorrelation(imag1, imag2)
	dx = dx * ((bx2-bx1)/(resolution-1))
	dy = dy * ((by2-by1)/(resolution-1))

	if text_display:
		print dx, dy
		
	if plotting:
		plt.ion()
		plt.figure()
		plt.scatter(stars2[:,1],stars2[:,0],s=20)
		plt.scatter(stars1[:,1]+dy,stars1[:,0]+dx,s=20,marker='+',color='r')
		plt.grid()
		plt.title("star matching after translation")
		plt.draw()

	return dx, dy

def approximate_astrometry(stars1, stars2, start_param = [1.,0.,0.,0.], nb_iterations=1, resolution=512, text_display = False, plotting = False):
	"""
	---------------------
	Purpose
	Calculate an estimate of the linear transformation law that allows to match two lists of star positions.
	The law has 4 coefficients (scaling, phi, dx, dy) and is defined by X2=scaling*(X1*cos(phi)-Y1*sin(phi))+dx, Y2=scaling*(X1*sin(phi)+Y1*cos(phi))+dy.
	A starting estimate of the 4 coefficients can be provided. The function makes use of astrometry.find_rotation_and_scaling() and astrometry.find_translation().
	---------------------
	Inputs
	* stars1 (2D Numpy array) = first list of star positions. stars1[:,0] corresponds to the X (vertical) coordinates and stars1[:,1] to the Y (horizontal) coordinates.
	* stars2 (2D Numpy array) = second list of star positions, with the same structure as stars1.
	* start_param (list of 4 float numbers) = optional parameter, default value is [1.,0.,0.,0.]. Starting estimates of the 4 parameters of the law, ordered as follows: [scaling, phi, dx, dy].
	* nb_iterations (integer) = optional parameter, default value is 1. This is the number of iterations to be used. One iteration corresponds to one call of astrometry.find_rotation_and_scaling() followed by one call of astrometry.find_translation().
	* resolution (integer) = optional parameter, default value is 512. This is the size in pixels of the artificial square images that are created and Fourier-transformed to calculate a cross-correlation.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	* plotting (boolean) = optional argument. If equal to True (default), the script is paused to display a plot showing the matching between the two lists of star positions. If equal to False no plot is shown.
	---------------------
	Outputs = scaling_final, phi_final, dx_final, dy_final
	These 4 coefficients define the linear transformation. For all the provided positions i we have:
		scaling_final*(stars1[i,0]*cos(phi_final)-stars1[i,1]*sin(phi_final))+dx_final approximately equal to stars2[i,0]
		scaling_final*(stars1[i,0]*sin(phi_final)+stars1[i,1]*cos(phi_final))+dy_final approximately equal to stars2[i,1]
	---------------------
	"""

	scaling_final = start_param[0]
	phi_final = start_param[1]
	dx_final = start_param[2]
	dy_final = start_param[3]

	for i in range(nb_iterations):

		#--- rotation + scaling ---
		phi, scaling = find_rotation_and_scaling(stars1, stars2, resolution=resolution, text_display=text_display, plotting=plotting)

		#--- converting results ---
		stars3 = []
		for st in range(np.shape(stars1)[0]):
			x = scaling*(stars1[st,0]*np.cos(phi) - stars1[st,1]*np.sin(phi))
			y = scaling*(stars1[st,0]*np.sin(phi) + stars1[st,1]*np.cos(phi))
			stars3.append([x,y])
		stars3 = np.array(stars3)
		stars4 = stars2

		#--- translation ---
		dx, dy = find_translation(stars3, stars4, resolution=resolution, text_display=text_display, plotting=plotting)

		#--- converting results ---
		stars1 = []
		for st in range(np.shape(stars3)[0]):
			x = stars3[st,0] + dx
			y = stars3[st,1] + dy
			stars1.append([x,y])
		stars1 = np.array(stars1)
		stars2 = stars4

		scaling_final = scaling_final * scaling
		phi_final = phi_final + phi
		dx_final2 = dx + scaling*np.cos(phi)*dx_final - scaling*np.sin(phi)*dy_final
		dy_final2 = dy + scaling*np.sin(phi)*dx_final + scaling*np.cos(phi)*dy_final
		dx_final = dx_final2
		dy_final = dy_final2
		
	return scaling_final, phi_final, dx_final, dy_final
