# -*- coding: iso_8859_1 -*-

from scipy import optimize
import numpy as np

def distorsion_law(p0, xy, scaling = np.ones(20)):
	"""
	---------------------
	Purpose
	Transform pixel coordinates (x,y) into projected stereographic coordinates (pv, ph).
	The 20 transformation coefficients that describe a 3rd order polynomial distorsion law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the 20 transformation coefficients.
	* xy (Numpy array) = 2D array such that xy[:,0] corresponds to the X (vertical) pixel coordinates and xy[:,1] to the Y (horizontal) pixel coordinates.
	* scaling (Numpy array with 20 elements) = optional argument, default value is np.ones(20). Allows to scale the transformation coefficients individually so that they have the same order of magnitude to improve numerical accuracy. For instance, scaling = np.array([1e5,1.,1.,1e-6,1e-6,1e-6,1e-9,1e-9,1e-9,1e-9]*2) allows to replace p[0]=104545, p[3]=1.1e-6 with p[0]=1.04545, p[1]=1.1, etc.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = vertical projected coordinates, Output[:,1] = horizontal projected coordinates.
	---------------------
	"""
	p = p0 * scaling
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1] + p[3]*(xy[:,0]**2.) + p[4]*xy[:,0]*xy[:,1] + p[5]*(xy[:,1]**2.) +\
					p[6]*(xy[:,0]**3.) + p[7]*(xy[:,0]**2.)*xy[:,1] + p[8]*xy[:,0]*(xy[:,1]**2.) + p[9]*(xy[:,1]**3.), \
					p[10] + p[11]*xy[:,0] + p[12]*xy[:,1] + p[13]*(xy[:,0]**2.) + p[14]*xy[:,0]*xy[:,1] + p[15]*(xy[:,1]**2.) +\
					p[16]*(xy[:,0]**3.) + p[17]*(xy[:,0]**2.)*xy[:,1] + p[18]*xy[:,0]*(xy[:,1]**2.) + p[19]*(xy[:,1]**3.)]).transpose()
					
def distorsion_law_quadratic(p, xy):
	"""
	---------------------
	Purpose
	Transform pixel coordinates (x,y) into projected stereographic coordinates (pv, ph).
	The 12 transformation coefficients that describe a 2nd order polynomial distorsion law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the 12 transformation coefficients.
	* xy (Numpy array) = 2D array such that xy[:,0] corresponds to the X (vertical) pixel coordinates and xy[:,1] to the Y (horizontal) pixel coordinates.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = vertical projected coordinates, Output[:,1] = horizontal projected coordinates.
	---------------------
	"""
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1] + p[3]*(xy[:,0]**2.) + p[4]*xy[:,0]*xy[:,1] + p[5]*(xy[:,1]**2.), \
					p[6] + p[7]*xy[:,0] + p[8]*xy[:,1] + p[9]*(xy[:,0]**2.) + p[10]*xy[:,0]*xy[:,1] + p[11]*(xy[:,1]**2.)]).transpose()
					
def distorsion_law_linear(p, xy):
	"""
	---------------------
	Purpose
	Transform pixel coordinates (x,y) into projected stereographic coordinates (pv, ph).
	The 6 transformation coefficients that describe a linear law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the 6 transformation coefficients.
	* xy (Numpy array) = 2D array such that xy[:,0] corresponds to the X (vertical) pixel coordinates and xy[:,1] to the Y (horizontal) pixel coordinates.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = vertical projected coordinates, Output[:,1] = horizontal projected coordinates.
	---------------------
	"""
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1], p[3] + p[4]*xy[:,0] + p[5]*xy[:,1]]).transpose()

def inverse_distorsion_law(p, proj, scaling = np.ones(20)):
	"""
	---------------------
	Purpose
	Transform projected stereographic coordinates (pv, ph) into pixel coordinates (x,y).
	The 20 transformation coefficients that describe a 3rd order polynomial distorsion law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the transformation coefficients.
	* proj (Numpy array) = 2D array such that proj[:,0] corresponds to the vertical projected coordinates and proj[:,1] to the horizontal projected coordinates.
	* scaling (Numpy array with 20 elements) = optional argument, default value is np.ones(20). Allows to scale the transformation coefficients individually so that they have the same order of magnitude to improve numerical accuracy. For instance, scaling = np.array([1e5,1.,1.,1e-6,1e-6,1e-6,1e-9,1e-9,1e-9,1e-9]*2) allows to replace p[0]=104545, p[3]=1.1e-6 with p[0]=1.04545, p[1]=1.1, etc.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = X (vertical) pixel coordinates, Output[:,1] = Y (horizontal) pixel coordinates.
	---------------------
	"""
	residual = lambda xy0,proj0,scaling: distorsion_law(p,np.tile(xy0,(1,1)),scaling=scaling)[0,:]-proj0
	
	xy = np.array(map(lambda proj0: optimize.fsolve(residual,[0.,0.],args=(proj0,scaling,)), proj))
	return xy
					
def inverse_distorsion_law_quadratic(p, proj):
	"""
	---------------------
	Purpose
	Transform projected stereographic coordinates (pv, ph) into pixel coordinates (x,y).
	The 12 transformation coefficients that describe a 2nd order polynomial distorsion law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the transformation coefficients.
	* proj (Numpy array) = 2D array such that proj[:,0] corresponds to the vertical projected coordinates and proj[:,1] to the horizontal projected coordinates.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = X (vertical) pixel coordinates, Output[:,1] = Y (horizontal) pixel coordinates.
	---------------------
	"""
	residual = lambda xy0,proj0: distorsion_law_quadratic(p,np.tile(xy0,(1,1)))[0,:]-proj0
	
	xy = np.array(map(lambda proj0: optimize.fsolve(residual,[0.,0.],args=(proj0)), proj))
	return xy

def inverse_distorsion_law_linear(p, proj):
	"""
	---------------------
	Purpose
	Transform projected stereographic coordinates (pv, ph) into pixel coordinates (x,y).
	The 6 transformation coefficients that describe a linear law must be provided.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the transformation coefficients.
	* proj (Numpy array) = 2D array such that proj[:,0] corresponds to the vertical projected coordinates and proj[:,1] to the horizontal projected coordinates.
	---------------------
	Output (Numpy array) = 2D array such that: Output[:,0] = X (vertical) pixel coordinates, Output[:,1] = Y (horizontal) pixel coordinates.
	---------------------
	"""
	residual = lambda xy0,proj0: distorsion_law_linear(p,np.tile(xy0,(1,1)))[0,:]-proj0
	
	xy = np.array(map(lambda proj0: optimize.fsolve(residual,[0.,0.],args=(proj0)), proj))
	return xy

def invert_simple_coefficients(p):
	"""
	---------------------
	Purpose
	Invert the coefficients of a simple linear transformation law that applies on pixel coordinates.
	The law to be inverted is defined by its offset, scaling and rotation coefficients.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the transformation coefficients. 
	The law coefficients are:
	p[0] = scaling factor, 
	p[1] = rotation angle phi in radians, 
	p[2] = dx = offset in X (vertical) direction, 
	p[3] = dy = offset in Y (horizontal) direction.
	The transformation law is defined by: 
	X2 = scaling*(X1*cos(phi) - Y1*sin(phi)) + dx
	Y2 = scaling*(X1*sin(phi) + Y1*cos(phi)) + dy
	---------------------
	Output (Numpy array) = inverted coefficients.
	---------------------
	"""	
	scaling, phi, dx, dy = p
	
	scaling2 = 1./scaling
	phi2 = - phi
	dx2 = - scaling2 * (np.cos(phi)*dx + np.sin(phi)*dy)
	dy2 = - scaling2 * (-np.sin(phi)*dx + np.cos(phi)*dy)
	
	return [scaling2, phi2, dx2, dy2]

def transform_stars(p, xy):
	"""
	---------------------
	Purpose
	Apply a simple linear transformation law on pixel coordinates.
	The law is defined by its offset, scaling and rotation coefficients.
	---------------------
	Inputs
	* p (Numpy array) = array that contains the transformation coefficients. 
	The law coefficients are:
	p[0] = scaling factor, 
	p[1] = rotation angle phi in radians, 
	p[2] = dx = offset in X (vertical) direction, 
	p[3] = dy = offset in Y (horizontal) direction.
	The transformation law is defined by: 
	X2 = scaling*(X1*cos(phi) - Y1*sin(phi)) + dx
	Y2 = scaling*(X1*sin(phi) + Y1*cos(phi)) + dy
	* xy (Numpy array) = 2D array such that xy[:,0] corresponds to the X (vertical) pixel coordinates and xy[:,1] to the Y (horizontal) pixel coordinates.
	---------------------
	Output (Numpy array) = array of transformed pixel coordinates, with same shape as the input array.
	---------------------
	"""	

	scaling, phi, dx, dy = p
	trans = map(lambda coord: [scaling*(coord[0]*np.cos(phi) - coord[1]*np.sin(phi)) + dx, \
							   scaling*(coord[0]*np.sin(phi) + coord[1]*np.cos(phi)) + dy],xy)
	return np.array(trans)

