# -*- coding: iso_8859_1 -*-

import numpy as np
from scipy import optimize

def fit_givenperiod(alldata, period, T0, nb_harmo = 6, fit_offsets = False):
	"""
	---------------------
	Purpose
	Performs a least square fit of a lightcurve with Fourier series for a user-specified period, and returns the corresponding rms magnitude error.
	This function can be used to plot a curve showing the fit error as a function of period, and to search for the most likely period.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays:
	- each Numpy array in the list has 3 columns and contains one data set,
	- the first column contains Julian dates of the observations, with all time corrections been made
	(the Julian date must be mid-exposure times, and the time correction for light propagation delays must have been made with cdr.time_correction() before calling the function),
	- the second one contains the magnitudes,
	- the third one contains an estimation of the magnitude noise for each magnitude value (not used in this function),
	- one array can contain several nights of observation, but must correspond to a single set of comparison stars.
	* period (float) = user specified period used for the least square fit. It is a constant parameter and will not be optimised.
	* T0 (float) = Reference Julian date. This quantity is subtracted from all the Julian dates used in the least square fit calculation, to avoid large numbers.
	* nb_harmo (integer) = optional parameter, default value is 6. Number of Fourier harmonics used for the fit. If F is the fundamental frequency, then the highest frequency used for the fit is nb_harmo*F (6*F by default).
	* fit_offsets (boolean) = if True, offsets are added to each data set separately during and fitted to the best values. This functionality can be used to fit asteroid lightcurves, when different reference stars are used for each data set. If False, the same offset is added to all data sets.
	---------------------
	Output (float) = square of rms magnitude error.
	---------------------
	"""

	print "... least square fit with Fourier series, period = ",period 

	#fit parameters
	an = np.ones((nb_harmo))
	bn = np.ones((nb_harmo))
	if fit_offsets:
		offsets = np.zeros((len(alldata)))
	else:
		offsets = np.zeros((1))
	p = list(an) + list(bn) + list(offsets)
		
	#fit function
	def CDR(alldata, *p):
		#global T0
		result = np.array([])
		for i,dd in enumerate(alldata):
			arg = (dd[:,0]-T0)*2.*np.pi/period
			if fit_offsets:
				fit = p[i+2*nb_harmo]
			else:
				fit = p[2*nb_harmo]
			for j in range(nb_harmo):
				fit = fit + p[j]*np.cos(np.double(j+1)*arg) + p[j+nb_harmo]*np.sin(np.double(j+1)*arg)
			result = np.hstack((result,np.array(fit)))
		return result
		
	#re-organizing measurement data
	ydata = np.zeros((0.))
	for i,dd in enumerate(alldata):
		ydata = np.hstack((ydata,dd[:,1]))

	#fitting
	popt, pcov = optimize.curve_fit(CDR, alldata, ydata, p0=p)

	#calculating squared error
	return np.sum(((ydata - CDR(alldata, *popt))**2.).flatten())
