# -*- coding: iso_8859_1 -*-

import numpy as np
from scipy import optimize
from aspylib import conf

def fit_CDR_fitperiod(alldata, period, T0, nb_harmo = 6, show_results = True, fit_offsets = False):
	"""
	---------------------
	Purpose
	Performs a least square fit of lightcurve with Fourier series.
	Returns the magnitude amplitude, Julian date of maximum and minimum magnitudes.
	Also returns all the fit parameters and their covariance matrix.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays:
	- each Numpy array in the list has 3 columns and contains one data set,
	- the first column contains Julian dates of the observations, with all time corrections been made
	(the Julian date must be mid-exposure times, and the time correction for light propagation delays must have been made with cdr.time_correction() before calling the function),
	- the second one contains the magnitudes,
	- the third one contains an estimation of the magnitude noise for each magnitude value,
	- one array can contain several nights of observation, but must correspond to a single set of comparison stars.
	* period (float) = starting period value used for the least square fit.
	* T0 (float) = Reference Julian date. This quantity is subtracted from all the Julian dates used in the least square fit calculation, to avoid large numbers.
	The returned Julian dates for min and max magnitudes are the first min and max dates after this reference Julian date.
	* nb_harmo (integer) = optional parameter, default value is 6. Number of Fourier harmonics used for the fit. If F is the fundamental frequency, then the highest frequency used for the fit is nb_harmo*F (6*F by default).
	* show_results (boolean) = if True, then all output parameters are displayed in the Python console. If False, nothing is displayed.
	* fit_offsets (boolean) = if True, offsets are added to each data set separately during and fitted to the best values. This functionality can be used to fit asteroid lightcurves, when different reference stars are used for each data set. If False, the same offset is added to all data sets.
	---------------------
	Outputs = popt, pcov, period, dperiod, amplitude, maxdate, mindate
	* popt (Numpy array) = this array contains all the fit parameters.
	If the input list of magnitudes (alldata) contains P data sets with different magnitude offsets, and if nb_harmo = N then the Numpy array has (1+2*N+P) elements (or 1+2*N elements if fit_offsets=False). The elements are:
	- popt[0] is the fundamental period
	- popt[1:1+N] are the amplitudes of the cosine terms with frequency i*F = i/popt[0] with i=1,2,..,N
	- popt[1+N:1+2*N] are the amplitudes of the sine terms with frequency i*F = i/popt[0] with i=1,2,..,N
	- popt[1+2*N:1+2*N+P] are the magnitude offsets to be used for each serie of measured magnitudes in the alldata input
	* pcov (Numpy array) = this array is a square matrix of size (1+2*N+P)*(1+2*N+P) (or (1+2*N)*(1+2*N) if fit_offsets=False) equal to the covariance matrix obtained with the least square fit.
	The rms uncertainty of the fit parameter popt[k] is equal to np.sqrt(pcov[k,k]).
	* period (float) = fundamental period of the lightcurve in days.
	* dperiod (float) = rms uncertainty of period.
	* amplitude (float) = peak to peak amplitude of the lightcurve.
	* maxdate (float) = Julian date of the lightcurve maximum magnitude (weakest brightness).
	* mindate (float) = Julian date of the lightcurve minimum magnitude (strongest brightness).
	---------------------
	"""

	print
	print "------ least square fit with Fourier series ------" 

	#initialisation of fit parameters
	an = np.ones((nb_harmo))
	bn = np.ones((nb_harmo))
	if fit_offsets:
		offsets = np.zeros((len(alldata)))
	else:
		offsets = np.zeros((1))
	p = [period] + list(an) + list(bn) + list(offsets)

	#p[0] = period
	#p[i+1] = an with i=0,1,..,nb_harmo-1
	#p[i+1+nb_harmo] = bn with i=0,1,..,nb_harmo-1
	#p[i+1+2*nb_harmo] = offsetn with i=0,1,..,nb_data_sets-1

	#fit function
	def CDR(alldata, *p):
		#global T0
		result = np.array([])
		for i,dd in enumerate(alldata):
			arg = (dd[:,0]-T0)*2.*np.pi/p[0]
			if fit_offsets:
				fit = p[i+1+2*nb_harmo]
			else:
				fit = p[1+2*nb_harmo]
			for j in range(nb_harmo):
				fit += p[j+1]*np.cos(np.double(j+1)*arg) + p[j+1+nb_harmo]*np.sin(np.double(j+1)*arg)
			result = np.hstack((result,np.array(fit)))
		return result
		
	#re-organizing measurement data
	ydata = np.array([])
	sigmadata = np.array([])

	for i,dd in enumerate(alldata):
		ydata = np.hstack((ydata,dd[:,1]))
		sigmadata = np.hstack((sigmadata,dd[:,2]))

	#fitting
	popt, pcov = optimize.curve_fit(CDR, alldata, ydata, p0=p, sigma = sigmadata)

	#calculating period and period uncertainty
	period = popt[0]
	dperiod = np.sqrt(pcov[0,0])
	
	#calculating amplitude
	x = np.linspace(0.,1.,10000)
	y = 0.
	for j in range(nb_harmo):
		y = y + popt[j+1]*np.cos(np.double(j+1)*x*2.*np.pi) + popt[j+1+nb_harmo]*np.sin(np.double(j+1)*x*2.*np.pi)
	amplitude = np.max(y) - np.min(y)

	#instants of min and max
	minphase = x[np.where(y==np.min(y))]
	maxphase = x[np.where(y==np.max(y))]
	mindate = T0 + popt[0]*minphase
	maxdate = T0 + popt[0]*maxphase
		
	if show_results:
		print
		print "------ fit results ------"	
		print
		print "fit result=",popt
		print "covariance matrix=",pcov
		print
		print "period (days)=",popt[0],"+/-",dperiod
		print "period (hours)=",24.*popt[0],"+/-",24.*dperiod
		print "amplitude (mag)=",amplitude
		print "date of max magnitude=",maxdate
		print "date of min magnitude=",mindate

	return popt, pcov, period, dperiod, amplitude, maxdate, mindate
